@@ -4,25 +4,26 @@
 {-# LANGUAGE OverloadedStrings  #-}
 -- | Simplified interface for common HTTP client interactions. Tutorial
 -- available at
--- <https://github.com/commercialhaskell/jump/blob/master/doc/http-client.md>.
+-- <https://haskell-lang.org/library/http-client>
 --
--- Important note: 'Request' is an instance of 'IsString', and therefore
--- recommended usage is to turn on @OverloadedStrings@, e.g.
+-- Important note: 'H.Request' is an instance of 'Data.String.IsString', and
+-- therefore recommended usage is to turn on @OverloadedStrings@, e.g.
 --
--- @@@
--- {-# LANGUAGE OverloadedStrings #-}
--- import Network.HTTP.Simple
--- import qualified Data.ByteString.Lazy.Char8 as L8
---
--- main :: IO ()
--- main = httpLBS "http://example.com" >>= L8.putStrLn
--- @@@
+-- > {-# LANGUAGE OverloadedStrings #-}
+-- > import Network.HTTP.Simple
+-- > import qualified Data.ByteString.Lazy.Char8 as L8
+-- >
+-- > main :: IO ()
+-- > main = httpLBS "http://example.com" >>= L8.putStrLn
 module Network.HTTP.Simple
     ( -- * Perform requests
       httpLBS
+    , httpNoBody
     , httpJSON
     , httpJSONEither
     , httpSink
+    , httpSource
+    , withResponse
       -- * Types
     , H.Request
     , H.Response
@@ -54,7 +55,7 @@ module Network.HTTP.Simple
     , setRequestBodyFile
     , setRequestBodyURLEncoded
       -- ** Special fields
-    , setRequestIgnoreStatus
+    , H.setRequestIgnoreStatus
     , setRequestBasicAuth
     , setRequestManager
     , setRequestProxy
@@ -79,16 +80,16 @@ import Control.Monad.IO.Class (MonadIO, liftIO)
 import Data.Aeson (FromJSON (..), Value)
 import Data.Aeson.Parser (json')
 import qualified Data.Aeson.Types as A
-import qualified Data.Aeson.Encode as A
+import qualified Data.Aeson as A
 import qualified Data.Traversable as T
 import Control.Exception (throwIO, Exception)
 import Data.Typeable (Typeable)
 import qualified Data.Conduit as C
 import qualified Data.Conduit.Attoparsec as C
 import qualified Control.Monad.Catch as Catch
-import Data.Default.Class (def)
 import qualified Network.HTTP.Types as H
 import Data.Int (Int64)
+import Control.Monad.Trans.Resource (MonadResource)
 
 -- | Perform an HTTP request and return the body as a lazy @ByteString@. Note
 -- that the entire value will be read into memory at once (no lazy I\/O will be
@@ -100,6 +101,14 @@ httpLBS req = liftIO $ do
     man <- H.getGlobalManager
     H.httpLbs req man
 
+-- | Perform an HTTP request and ignore the response body.
+--
+-- @since 2.2.2
+httpNoBody :: MonadIO m => H.Request -> m (H.Response ())
+httpNoBody req = liftIO $ do
+    man <- H.getGlobalManager
+    H.httpNoBody req man
+
 -- | Perform an HTTP request and parse the body as JSON. In the event of an
 -- JSON parse errors, a 'JSONException' runtime exception will be thrown.
 --
@@ -151,6 +160,65 @@ httpSink req sink = do
         (\res -> bodyReaderSource (getResponseBody res)
             C.$$ sink (fmap (const ()) res))
 
+-- | Perform an HTTP request, and get the response body as a Source.
+--
+-- The second argument to this function tells us how to make the
+-- Source from the Response itself. This allows you to perform actions
+-- with the status or headers, for example, in addition to the raw
+-- bytes themselves. If you just care about the response body, you can
+-- use 'getResponseBody' as the second argument here.
+--
+-- @
+-- \{\-# LANGUAGE OverloadedStrings \#\-}
+-- import           Control.Monad.IO.Class       (liftIO)
+-- import           Control.Monad.Trans.Resource (runResourceT)
+-- import           Data.Conduit                 (($$))
+-- import qualified Data.Conduit.Binary          as CB
+-- import qualified Data.Conduit.List            as CL
+-- import           Network.HTTP.Simple
+-- import           System.IO                    (stdout)
+--
+-- main :: IO ()
+-- main =
+--     runResourceT
+--         $ httpSource "http://httpbin.org/robots.txt" getSrc
+--        $$ CB.sinkHandle stdout
+--   where
+--     getSrc res = do
+--         liftIO $ print (getResponseStatus res, getResponseHeaders res)
+--         getResponseBody res
+-- @
+--
+-- @since 2.2.1
+httpSource :: (MonadResource m, MonadIO n)
+           => H.Request
+           -> (H.Response (C.ConduitM i S.ByteString n ())
+                -> C.ConduitM i o m r)
+           -> C.ConduitM i o m r
+httpSource req withRes = do
+    man <- liftIO H.getGlobalManager
+    C.bracketP (H.responseOpen req man) H.responseClose
+        (withRes . fmap bodyReaderSource)
+
+-- | Perform an action with the given request. This employes the
+-- bracket pattern.
+--
+-- This is similar to 'httpSource', but does not require
+-- 'MonadResource' and allows the result to not contain a 'C.ConduitM'
+-- value.
+--
+-- @since 2.2.3
+withResponse :: (MonadIO m, Catch.MonadMask m, MonadIO n)
+             => H.Request
+             -> (H.Response (C.ConduitM i S.ByteString n ()) -> m a)
+             -> m a
+withResponse req withRes = do
+    man <- liftIO H.getGlobalManager
+    Catch.bracket
+        (liftIO (H.responseOpen req man))
+        (liftIO . H.responseClose)
+        (withRes . fmap bodyReaderSource)
+
 -- | Alternate spelling of 'httpLBS'
 --
 -- @since 2.1.10
@@ -255,7 +323,7 @@ setRequestBodyJSON x req =
     req { H.requestHeaders
             = (H.hContentType, "application/json; charset=utf-8")
             : filter (\(y, _) -> y /= H.hContentType) (H.requestHeaders req)
-        , H.requestBody = H.RequestBodyLBS $ A.encode $ A.toJSON x
+        , H.requestBody = H.RequestBodyLBS $ A.encode x
         }
 
 -- | Set the request body as a lazy @ByteString@
@@ -299,13 +367,6 @@ setRequestBodyFile = setRequestBody . HI.RequestBodyIO . H.streamFile
 setRequestBodyURLEncoded :: [(S.ByteString, S.ByteString)] -> H.Request -> H.Request
 setRequestBodyURLEncoded = H.urlEncodedBody
 
--- | Modify the request so that non-2XX status codes do not generate a runtime
--- exception.
---
--- @since 2.1.10
-setRequestIgnoreStatus :: H.Request -> H.Request
-setRequestIgnoreStatus req = req { H.checkStatus = \_ _ _ -> Nothing }
-
 -- | Set basic auth with the given username and password
 --
 -- @since 2.1.10
